/***********************************************************************

  storageFcns.c - SD card functions

  author    John Madsen
  revision  1.0
  date      2022-05-23


  history   1.0 initial release

***********************************************************************/

#include "storageFcns.h"
#include "systemFcns.h"
#include "ymodem.h"
#include "text.h"

#include "mytypes.h"
#include <string.h>
#include <sys/unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <fcntl.h>
#include "esp_vfs_fat.h"
#include "sdmmc_cmd.h"
#include "driver/sdmmc_host.h"
#include "ff.h"
#include "vfs_fat_internal.h"
#include "esp_system.h"
#include "nvs.h"
#include "nvs_flash.h"





typedef unsigned int size_t;


static void init_filesystem(void);
static void init_file(void);
static void init_nvs(void);


static wl_handle_t 			wl_handle;

static const char 			*Tag = "storage";
static const char 			*FileText = MOUNT_PATH"/text.txt";




/*******************************************************************

		Public Functions

 *******************************************************************/

void storageInit(void)
{

    init_nvs();
    init_filesystem();

    init_file();
}



int storageSendFile(char *filename)
{
    FILE 			*fp = NULL;
    int				trans_res=-1;
    char 			path[FILE_PATH_MAX];
	struct stat 	stats;
    int 			statok;


	snprintf(path, sizeof(path), MOUNT_PATH"/%s", filename);

    statok = stat(path, &stats);
	if (statok)
	{
		ESP_LOGE(Tag, "File not found");
		return -1;
	}

//	ESP_LOGI(Tag, "Send file %s.", filename);
//	storageLogWriteEvent(EL_DATA_TRANFER_INIT, SUCCESS);

	fp = fopen(path, "rb");
	if (fp)
	{
		printf("\r\nSending file \"%s\", please start YModem receive on host ...\r\n", path);
		trans_res = Ymodem_Transmit(filename, stats.st_size, fp);
		fclose(fp);
		printf("\r\n");
		if (trans_res == 0)
		{
			printf("Transfer complete.\n");
		}
		else
		{
			printf("Transfer complete, Error=%d\n", trans_res);
		}
	}
	else
	{
		printf("Error opening file \"%s\" for sending.\n", path);
	}

	return 0;
}






int storprintf(const char *fmt, ...)
{
    int result;
    va_list args;



    // always write to stdout
    va_start(args, fmt);
    result = vprintf(fmt, args);


    va_end(args);
    return result;
}




/*******************************************************************

		Private Functions

 *******************************************************************/

static void init_filesystem(void)
{
	esp_err_t ret;
    const esp_vfs_fat_mount_config_t mount_config =
    {
        .max_files = 4,
        .format_if_mount_failed = true
    };


    ESP_LOGI(Tag, "Mounting spiflash filesystem");
    ret = esp_vfs_fat_spiflash_mount(MOUNT_PATH, "storage", &mount_config, &wl_handle);

    if (ret == ESP_OK)
    	ESP_LOGI(Tag, "spiflash Filesystem mounted");
    else
    	ESP_LOGE(Tag, "Failed to mount spiflash filesystem (%s)", esp_err_to_name(ret));
}


static void init_nvs(void)
{
    esp_err_t err = nvs_flash_init();
    if (err == ESP_ERR_NVS_NO_FREE_PAGES || err == ESP_ERR_NVS_NEW_VERSION_FOUND) {
        ESP_ERROR_CHECK( nvs_flash_erase() );
        err = nvs_flash_init();
    }
    ESP_ERROR_CHECK(err);
}


static void init_file(void)
{
	struct stat 	stats;
    int 			statok;


    statok = stat(FileText, &stats);
	if (statok == 0)
		unlink(FileText);

//	if (statok)
	{
		ESP_LOGI(Tag, "Create text file");

		FILE *fp = fopen(FileText, "w");
		if (fp == NULL) {
			ESP_LOGE(Tag, "Failed to open file for writing");
			return;
		}
		fprintf(fp, text, wl_handle);
		fclose(fp);
		ESP_LOGI(Tag, "File written");
	}
//	else
//		ESP_LOGI(Tag, "Text file exists");


}







